package com.shcd.boat.service.impl;

import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.shanghaigeography.entity.ResultMsg;
import com.shanghaigeography.eum.ResultState;
import com.shcd.boat.entity.HaiKangListVo;
import com.shcd.boat.entity.HaiKangOnLineVo;
import com.shcd.boat.entity.HaiKangSettingVo;
import com.shcd.boat.entity.ShorePowerFile;
import com.shcd.boat.service.HaiKangService;
import com.shcd.boat.utils.HaikangUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.bytedeco.javacv.FFmpegFrameGrabber;
import org.bytedeco.javacv.Frame;
import org.bytedeco.javacv.Java2DFrameConverter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 类名:@Name - HiKangServiceimpl
 * 功能描述:@Function Description<>
 * 日期:@Date - 2025/3/4 15:37
 */
@Service
@Slf4j
public class HiKangServiceimpl implements HaiKangService {

    @Value("${hiakangvideo}")
    private String hiakangvideo;
    @Value("${hiakangAppKey}")
    private String hiakangAppKey;
    @Value("${hiakangAppSecret}")
    private String hiakangAppSecret;
    @Value("${XCaSignature}")
    private String hiakangSignature;

    /**
     * 获取区域列表的资源
     * @return
     */
    @Override
    public String getList(Integer pageNo, Integer pageSize) {
        String url="/api/resource/v1/regions";
        JSONObject jsonObject=new JSONObject();
        jsonObject.put("pageNo",pageNo);
        jsonObject.put("pageSize",pageSize);
        JSONObject result= HaikangUtils.post(url, jsonObject.toJSONString(),null,null,null);
        if (!"0".equals(result.getString("code")))
        {
            return new ResultMsg<String>().setCode(ResultState.ERROR).setMsg(result.toJSONString()).toJSON();
        }
        List<HaiKangListVo> haiKangListVos=result.getJSONObject("data").getJSONArray("list").toJavaList(HaiKangListVo.class);
        List<HaiKangListVo> root =haiKangListVos.stream().filter(i->"-1".equals(i.getParentIndexCode())).collect(Collectors.toList());
        if (root!=null && root.size()>0)
        {
            //获取到区域在线的设备个数
            JSONObject objectOnLine= countOnLineSize(pageSize,root.get(0).getIndexCode(),"1");
            log.info("===   在线的设备"+objectOnLine.toString());
            JSONObject objectOffLine= countOnLineSize(pageSize,root.get(0).getIndexCode(),"0");
            log.info("===   不在线的设备"+objectOffLine.toString());
            haiKangListVos.stream().forEach(i->{
                i.setIsLeaf(false);
            });
            log.info("===   映射个数"+haiKangListVos.toString());
            getLevelList(haiKangListVos,root,objectOnLine,objectOffLine);
        }
        log.info("组装数据："+root);

        return new ResultMsg<String>().setCode(ResultState.SUCCESS).setData(JSONArray.of(root).toString()).toJSON();
    }

    /**
     * 统计区域视频在线个数，返回父Index统计
     * @param pageSize
     * @param code
     * @param status
     * @return
     */
    JSONObject countOnLineSize(Integer pageSize, String code, String status)
    {
        JSONObject result=new JSONObject();
        JSONObject resultOnLine= getCameraOnLine(1,pageSize,code,status);
        if ("0".equals(resultOnLine.getString("code")))
        {
            Integer allPages=resultOnLine.getJSONObject("data").getInteger("totalPage");
            List<HaiKangOnLineVo> haiKangListVos = resultOnLine.getJSONObject("data").getJSONArray("list").toJavaList(HaiKangOnLineVo.class);
            Map<String, List<HaiKangOnLineVo>> group = haiKangListVos.stream().collect(Collectors.groupingBy(HaiKangOnLineVo::getRegionIndexCode));
            group.keySet().stream().forEach(i -> {
                if (group.get(i)!=null) {
                    result.put(i, group.get(i).size());
                }else {
                    result.put(i, 0);
                }
            });
            if (allPages>1) {
                for (int i=2;i<=allPages;i++)
                {
                    JSONObject resultOnLine2= getCameraOnLine(i,pageSize,code,status);
                    if ("0".equals(resultOnLine2.getString("code"))) {
                        List<HaiKangOnLineVo> haiKangListVos2 = resultOnLine2.getJSONObject("data").getJSONArray("list").toJavaList(HaiKangOnLineVo.class);
                        Map<String, List<HaiKangOnLineVo>> group2 = haiKangListVos2.stream().collect(Collectors.groupingBy(HaiKangOnLineVo::getRegionIndexCode));
                        group2.keySet().stream().forEach(j -> {
                            if (result.getInteger(j)==null || result.getInteger(j)==0) {
                                if (group2.get(j)!=null) {
                                    result.put(j, group2.get(j).size());
                                }else {
                                    result.put(j, 0);
                                }
                            }else {

                                result.put(j,  result.getInteger(j)+group2.get(j).size());
                            }
                        });
                    }
                }
            }
        }
        return result;
    }

    /**
     * 返回在线集合
     * @param pageSize
     * @param code
     * @param status
     * @return
     */
    Set<String> countOnLineSlefSize(Integer pageSize, String code, String status)
    {
        Set<String> result=new HashSet<>();
        JSONObject resultOnLine= getCameraOnLine(1,pageSize,code,status);
        if ("0".equals(resultOnLine.getString("code")))
        {
            Integer allPages=resultOnLine.getJSONObject("data").getInteger("totalPage");
            List<HaiKangOnLineVo> haiKangListVos = resultOnLine.getJSONObject("data").getJSONArray("list").toJavaList(HaiKangOnLineVo.class);
            haiKangListVos.stream().forEach(i->{
                result.add(i.getIndexCode());
            });
            if (allPages>1) {
                for (int i=2;i<=allPages;i++)
                {
                    JSONObject resultOnLine2= getCameraOnLine(i,pageSize,code,status);
                    if ("0".equals(resultOnLine2.getString("code"))) {
                        List<HaiKangOnLineVo> haiKangListVos2 = resultOnLine2.getJSONObject("data").getJSONArray("list").toJavaList(HaiKangOnLineVo.class);
                        haiKangListVos2.stream().forEach(j->{
                            result.add(j.getIndexCode());
                        });
                    }
                }
            }
        }
        return result;
    }




    /**
     * 获取某个视频资源
     * @param code
     * @return
     */
    @Override
    public String getDetail(Integer pageNo, Integer pageSize, String code) {
        String url="/api/resource/v2/camera/search";
        JSONObject jsonObject=new JSONObject();
        jsonObject.put("pageNo",pageNo);
        jsonObject.put("isSubRegion",false);
        JSONArray jsonArray=new JSONArray();
        jsonArray.add(code);
        jsonObject.put("regionIndexCodes",jsonArray);
        jsonObject.put("pageSize",pageSize);
        JSONObject result= HaikangUtils.post(url, jsonObject.toJSONString(),null,null,null);
        if (!"0".equals(result.getString("code")))
        {
            return new ResultMsg<String>().setCode(ResultState.SUCCESS).setMsg(result.toJSONString()).toJSON();
        }
        List<HaiKangListVo> haiKangListVos=result.getJSONObject("data").getJSONArray("list").toJavaList(HaiKangListVo.class);
        if (haiKangListVos!=null && haiKangListVos.size()>0)
        {
            log.info("===   数据："+haiKangListVos.toString());
            Set<String> codes= countOnLineSlefSize(pageSize,code,"0");
            if (codes!=null && codes.size()>0) {
                log.info("===   数据在线情况："+codes.toString());
                haiKangListVos.stream().forEach(i->{
                    if (codes.contains(i.getIndexCode()))
                    {
                        i.setOnLine("no");
                    }
                });
            }

            return new ResultMsg<String>().setCode(ResultState.SUCCESS).setData(JSONArray.of(haiKangListVos).toString()).toJSON();
        }
        return new ResultMsg<String>().setCode(ResultState.SUCCESS).toJSON();


    }

    /**
     * 获取视频信息
     * @return
     */
    @Override
    public String getSetting() {
        if (StringUtils.isNotBlank(hiakangvideo)&& StringUtils.isNotBlank(hiakangAppKey)&& StringUtils.isNotBlank(hiakangAppSecret))
        {
            String[]  ipSetting=hiakangvideo.split(":");
            HaiKangSettingVo data= HaiKangSettingVo.builder().appkey(hiakangAppKey).ip(ipSetting[0]).port(Integer.valueOf(ipSetting[1])).secret(hiakangAppSecret).build();
            return new ResultMsg<String>().setCode(ResultState.SUCCESS).setData(JSONObject.toJSONString(data)).toJSON();
        }
        return new ResultMsg<String>().setCode(ResultState.SUCCESS).toJSON();
    }

    @Override
    public String getOnLine(Integer pageNo, Integer pageSize, String code) {
        JSONObject result= getCameraOnLine(pageNo,pageSize,code,"1");
        if (!"0".equals(result.getString("code")))
        {
            return new ResultMsg<String>().setCode(ResultState.SUCCESS).setMsg(result.toJSONString()).toJSON();
        }
        List<HaiKangOnLineVo> haiKangListVos=result.getJSONObject("data").getJSONArray("list").toJavaList(HaiKangOnLineVo.class);
        if (haiKangListVos!=null && haiKangListVos.size()>0)
        {
            return new ResultMsg<String>().setCode(ResultState.SUCCESS).setData(JSONArray.of(haiKangListVos).toString()).toJSON();
        }
        return new ResultMsg<String>().setCode(ResultState.SUCCESS).toJSON();
    }

    @Override
    public String getSocketURL(JSONObject jsonObject) {
        String url="/api/video/v1/cameras/previewURLs";
        Map<String, String> header = new HashMap<>();
        header.put("X-Ca-Key",hiakangAppKey);
        header.put("X-Ca-Signature-Headers", "x-ca-key");
        header.put("Content-Type", "application/json");
        header.put("X-Ca-Signature", hiakangSignature);

        JSONObject result= HaikangUtils.post(url, jsonObject.toJSONString(),null,null,header);
        String urlVideo = "";
        if (!ObjectUtils.isEmpty(result)){
            urlVideo = result.getJSONObject("data").get("url").toString();
        }
        return  urlVideo;
    }

    /**
     * 功能描述: <获取进港拍照截图>
     * 方法名：@Name - doInPicture
     * 参数列表：@Param - [java.util.List<java.lang.String>]
     * 返回值类型：@Return - java.util.List<com.shcd.boat.entity.ShorePowerFile>
     * 作者：@Author - zrj
     * 日期：@Date - 2025/3/21 15:21
     */
    @Override
    public List<ShorePowerFile> doInPicture(List<String> cameraIndexCodes) {
        if (!CollectionUtils.isEmpty(cameraIndexCodes)){
            List<ShorePowerFile> shorePowerFileList = new ArrayList<>();
            ShorePowerFile shorePowerFile = new ShorePowerFile();
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("protocol","hls");
            jsonObject.put("streamType",0);
            jsonObject.put("transmode",0);
            cameraIndexCodes.forEach(i -> {
                    byte[] bytes = this.getPicture(i,jsonObject,baos);
                    shorePowerFile.setInImageData(bytes);
                    shorePowerFile.setType("jpg");
                    shorePowerFile.setInImageStatus("1");
                    shorePowerFile.setOutImageStatus("0");
                    shorePowerFileList.add(shorePowerFile);
            });
            return shorePowerFileList;
        }
        return null;
    }

    @Override
    public List<ShorePowerFile> doOutPicture(List<String> cameraIndexCodes) {
        if (!CollectionUtils.isEmpty(cameraIndexCodes)){
            List<ShorePowerFile> shorePowerFileList = new ArrayList<>();
            ShorePowerFile shorePowerFile = new ShorePowerFile();
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("protocol","hls");
            jsonObject.put("streamType",0);
            jsonObject.put("transmode",0);
            cameraIndexCodes.forEach(i -> {
                byte[] bytes = this.getPicture(i,jsonObject,baos);
                shorePowerFile.setOutImageData(bytes);
                shorePowerFile.setType("jpg");
                shorePowerFile.setOutImageStatus("1");
                shorePowerFileList.add(shorePowerFile);
            });
            return shorePowerFileList;
        }
        return null;
    }

    private byte[] getPicture(String cameraIndexCode,JSONObject jsonObject,ByteArrayOutputStream baos) {
        try {
            jsonObject.put("cameraIndexCode", cameraIndexCode);
            String socketURL = this.getSocketURL(jsonObject);
            log.info("海康视频流:" + socketURL);
            FFmpegFrameGrabber grabber = FFmpegFrameGrabber.createDefault(socketURL);
            Java2DFrameConverter converter = new Java2DFrameConverter();
            grabber.start();

            Frame frame = grabber.grabImage();
            if (frame == null) {
                log.info("未获取到有效帧！");
                return null;
            }
            // 生成带时间戳的文件名
            BufferedImage image = converter.getBufferedImage(frame);
            ImageIO.write(image, "jpg", baos);
            baos.flush();
            byte[] bytes = baos.toByteArray();
            baos.close();
            return bytes;
        } catch (Exception e) {
            log.info(e.getMessage());
        }
        return null;
    }


    /**
     *
     * @param pageNo
     * @param pageSize
     * @param code
     *  @param status        状态，1-在线，0-离线，-1-未检测
     * @return
     */
    JSONObject  getCameraOnLine(Integer pageNo, Integer pageSize, String code, String status)
    {
        String url="/api/nms/v1/online/camera/get";
        JSONObject jsonObject=new JSONObject();
//        regionId	string	False	区域id；
        jsonObject.put("regionId",code);
//        includeSubNode	string	False	是否包含下级区域中的资源数据，1包含，0不包含（若regionId为空，则该参数不起作用
        jsonObject.put("includeSubNode","1");
//        status	string	False	状态，1-在线，0-离线，-1-未检测
//        indexCodes	string[]	False	监控点编号列表，最大500，
        jsonObject.put("status",status);
        jsonObject.put("pageNo",pageNo);
        jsonObject.put("pageSize",pageSize);
        JSONObject result= HaikangUtils.post(url, jsonObject.toJSONString(),null,null,null);
        return result;
    }




    /**
     * 级别划分
     * @param result
     * @param haiKangListVo
     * @return
     */
    List<HaiKangListVo> getLevelList(List<HaiKangListVo> result, List<HaiKangListVo> haiKangListVo , JSONObject objectOnLine, JSONObject objectOffLine){

        if (result!=null&&result.size()>0)
        {

            haiKangListVo.stream().forEach(i->{
                List<HaiKangListVo> haiKangListVos1=result.stream().filter(j->i.getIndexCode().equals(j.getParentIndexCode())).collect(Collectors.toList());
                if (haiKangListVos1!=null&&haiKangListVos1.size()>0)
                {
                    i.setListVos(haiKangListVos1);
                    i.setCameraOnLineCount(haiKangListVos1.stream().filter(j->j.getCameraCount()!=null).mapToInt(HaiKangListVo::getCameraOnLineCount).sum());
                    i.setCameraOffLineCount(haiKangListVos1.stream().filter(j->j.getCameraCount()!=null).mapToInt(HaiKangListVo::getCameraOffLineCount).sum());
                    if ((i.getCameraOnLineCount()+i.getCameraOffLineCount())>0) {
                        i.setCameraCount("(" +i.getCameraOnLineCount() +"/" + (i.getCameraOnLineCount()+i.getCameraOffLineCount()) + ")");
                    }
                    getLevelList(result,haiKangListVos1, objectOnLine,objectOffLine);
                    i.setCameraOnLineCount(haiKangListVos1.stream().filter(j->j.getCameraCount()!=null).mapToInt(HaiKangListVo::getCameraOnLineCount).sum());
                    i.setCameraOffLineCount(haiKangListVos1.stream().filter(j->j.getCameraCount()!=null).mapToInt(HaiKangListVo::getCameraOffLineCount).sum());
                    if ((i.getCameraOnLineCount()+i.getCameraOffLineCount())>0) {
                        i.setCameraCount("(" +i.getCameraOnLineCount() +"/" + (i.getCameraOnLineCount()+i.getCameraOffLineCount()) + ")");
                    }

                }else {
                    //最底层
                    i.setCameraOnLineCount((objectOnLine.getInteger(i.getIndexCode()) == null ? 0 : +objectOnLine.getInteger(i.getIndexCode())));
                    i.setCameraOffLineCount((objectOffLine.getInteger(i.getIndexCode()) == null ? 0 : +objectOffLine.getInteger(i.getIndexCode())));
                    if ((i.getCameraOnLineCount() + i.getCameraOffLineCount()) > 0) {
                        i.setCameraCount("(" + i.getCameraOnLineCount() + "/" + (i.getCameraOnLineCount() + i.getCameraOffLineCount()) + ")");
                    }
                }

            });


        }
        return haiKangListVo;
    }


}