package com.shcd.boat.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.shanghaigeography.Util.DateUtils;
import com.shcd.boat.config.RedisCache;
import com.shcd.boat.config.RedisUtilTemplate;
import com.shcd.boat.config.SnowConfig;
import com.shcd.boat.entity.*;
import com.shcd.boat.entity.gh.GHAisInfo;
import com.shcd.boat.entity.gh.PageForGHAisInfo;
import com.shcd.boat.entity.gh.ShipReportData;
import com.shcd.boat.mapper.ShipSealOffInfoMapper;
import com.shcd.boat.mapper.ShipSealOffPointsMapper;
import com.shcd.boat.mapper.ShipSealOffSpaceMapper;
import com.shcd.boat.service.IShipSealOffInfoService;
import com.shcd.boat.service.ShipSealOffService;
import com.shcd.boat.sms.MASSMSService;
import com.shcd.boat.utils.Encodes;
import com.shcd.boat.utils.GhjgRpcUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.concurrent.atomic.AtomicReference;
import java.util.function.Function;
import java.util.stream.Collectors;

@Slf4j
@Service
public class ShipSealOffServiceImpl implements ShipSealOffService {

    @Value("${transUrl}")
    private String  transUrl;
//    1.先获取AIS
//    2.再分析船舶是否在范围内
//    3.在范围内查看是否【缓存】已有点位  没有点位加，有点位小于2也加，大于2 且船速>1 开始预警
//    4.

    @Autowired
    private MASSMSService massmsService;
    /**
     * 获取市港航数据
     * @return
     */
    @Override
    public PageForGHAisInfo getGHAISData(JSONObject params, BigDecimal shipLength,BigDecimal shipBreadth) {
        PageForGHAisInfo ghAisInfo = new PageForGHAisInfo();
        String response = GhjgRpcUtils.getAISData(transUrl, params);
        JSONObject object = JSON.parseObject(response);
        JSONArray jsonArray = object.getJSONArray("rows");
        ghAisInfo.setPageIndex(object.getLong("pageIndex"));
        ghAisInfo.setPageSize(object.getLong("pageSize"));
        ghAisInfo.setTotal(object.getLong("total"));
        if (jsonArray != null && jsonArray.size() > 0) {
            List<GHAisInfo> aisInfos = new ArrayList<>(jsonArray.size());
            jsonArray.forEach(i -> {
                aisInfos.add(JSON.parseObject(i.toString(), GHAisInfo.class));
            });
            List<GHAisInfo> aisInfosFilter =aisInfos.stream()
                    .filter(i->i.getShipBreadth()!=null&&shipBreadth.compareTo(i.getShipBreadth())<=0)
                    .filter(i->i.getShipLength()!=null&&shipLength.compareTo(i.getShipLength())<=0)
//                    .filter(i->i.getVelocity()!=null&&new BigDecimal("1").compareTo(i.getVelocity())<=0)
                    .collect(Collectors.toList());
            log.info("===   本次返回条目数："+aisInfos.size()+"其中：符合船舶总长>"+shipLength+",船舶型宽>"+shipBreadth+"且船速大于1的船舶有："+aisInfosFilter.size());
            ghAisInfo.setDatas(aisInfosFilter);
            return ghAisInfo;
        }
        return ghAisInfo;
    }

    /**
     * 开始分析 获取符合条件的点位数据
     * @return
     */
    @Override
    public  List<GHAisInfo>  analysisData(BigDecimal maxX,BigDecimal minX,BigDecimal maxY,BigDecimal minY) {
        Integer pageSize=500;
        JSONObject params1 = new JSONObject();
        params1.put("pageIndex", 1);
        params1.put("pageSize", pageSize);
        params1.put("maxX", maxX);
        params1.put("minX",minX);
        String nowData=DateUtils.getNowTimeStringToDD(DateUtils.DATE_FORMAT_19);
        params1.put("maxY", maxY);
        params1.put("minY",minY);
        params1.put("startTime",DateUtils.addDateMinut(nowData,-30));
        params1.put("endTime",nowData);
        log.info("===    开始查询第：" + 1+ "页");
//        ExecutorService es = Executors.newFixedThreadPool(10);
        PageForGHAisInfo ghAisInfo= this.getGHAISData(params1,new BigDecimal("37"),new BigDecimal("7.2"));
         List<GHAisInfo> aisInfos=ghAisInfo.getDatas();
        log.info("===        本次获取符合条件的数据："+ghAisInfo.toString());
            if (ghAisInfo.getTotal() > pageSize) {
                log.info("===    数据个数：" + ghAisInfo.getTotal() + "需要页数：" + ((ghAisInfo.getTotal() - 1) / pageSize + 1));
                for (int i = 2; i <= ((ghAisInfo.getTotal() - 1) / pageSize + 1); i++) {
                    log.info("===    开始查询第：" + i+ "页");
                    params1.put("pageIndex", i);
                    params1.put("pageSize", pageSize);
                    PageForGHAisInfo ghAisInfos= this.getGHAISData(params1,new BigDecimal("37"),new BigDecimal("7.2"));
                   if (ghAisInfos.getDatas()!=null && ghAisInfos.getDatas().size()>0) {
                       aisInfos.addAll(ghAisInfos.getDatas());
                   }
                }
            }
            if (ghAisInfo.getDatas()!=null) {
                log.info("===      全部获取符合条件的数据个数为：" + aisInfos.size());
                return aisInfos;
            }else {
                return null;
            }
    }

    @Resource
    private ShipSealOffSpaceMapper shipSealOffSpaceMapper;
    @Autowired
    private SnowConfig snowflakeConfig;


    @Resource
    private ShipSealOffPointsMapper offPointsMapper;
    @Autowired
    private RedisCache redisUtilTemplate;

    /**
     * 分析点位
     */
    @Override
    public void analysisGISPointInData() throws InterruptedException {
        log.info("===  1 先查看缓存里的数据在数据库里数据状态是否为待确认，非待待确认的，缓存删除");
        getDBAllDataForNeed();

        log.info("===  2 开始查询市港航接口30分钟内符合条件的数据（长大于37,且宽度大于7.2)的数据");
        //          处理三个区域点位
        List<GHAisInfo> aisInfosAll=new ArrayList<>();
        List<GHAisInfo>  leftPoints=  analysisData(new BigDecimal("9710.84684148"),new BigDecimal("0.06495828"),new BigDecimal("-5650.65077473"),new BigDecimal("-8201.43027414"));
        if (leftPoints!=null&& leftPoints.size()>0)
        {
            log.info("==   2.1 左边区域符合点位个数："+leftPoints.size());
            aisInfosAll.addAll(leftPoints);
        }
        Thread.sleep(500);
        List<GHAisInfo>  centerPoints=  analysisData(new BigDecimal("21100.84684148"),new BigDecimal("9700.84684148"),new BigDecimal("8601.43027414"),new BigDecimal("-6050.65077473"));
        if (centerPoints!=null&& centerPoints.size()>0)
        {
            log.info("==   2.2 中心区域符合点位个数："+centerPoints.size());
            aisInfosAll.addAll(centerPoints);
        }
        Thread.sleep(500);
        List<GHAisInfo>  rightPoints=  analysisData(new BigDecimal("29500.8468414"),new BigDecimal("21090.84684148"),new BigDecimal("-100.43027414"),new BigDecimal("-3400.65077473"));
        if (rightPoints!=null&& rightPoints.size()>0)
        {
            log.info("==   2.3 右部区域符合点位个数："+rightPoints.size());
            aisInfosAll.addAll(rightPoints);
        }
        log.info("==   2.4 全部区域符合点位个数："+aisInfosAll.size());
       List<GHAisInfo> newAisInfosAll= shipSealOffSpaceMapper.getAllInOut(aisInfosAll);
        log.info("==   2.5 所在范围情况:"+newAisInfosAll.toString());
        List<GHAisInfo>  inAll=newAisInfosAll.stream().filter(i->i.getInAll()!=null && i.getInAll()).collect(Collectors.toList());
        //处理在范围内的船
        List<GHAisInfo>  notInAll=newAisInfosAll.stream().filter(i->i.getInAll()!=null && !i.getInAll()).collect(Collectors.toList());
       if (inAll!=null && inAll.size()>0)
       {
           log.info("==  3.处理在区域范围内的船舶个数："+inAll.size());
           saveAllInData(inAll);
       }
       if (notInAll!=null && notInAll.size()>0)
       {
           log.info("==  4.处理不在区域范围内的船舶个数："+notInAll.size());
           saveAllNotData(notInAll);
       }




    }

    @Autowired
    private IShipSealOffInfoService shipSealOffInfoService;
    /**
     * 判断本次点位在哪个航道
     * @return
     */
    String getInChannelForThisAIS(GHAisInfo thisAisPoint)
    {
        AISInChannel aisInChannel= shipSealOffSpaceMapper.getInChannel(thisAisPoint);
        if(aisInChannel!=null)
        {
             if (aisInChannel.getChuangyanghe())
             {
                 return "川杨河";
             }else if (aisInChannel.getPudongyunhe())
             {
                 return "浦东运河(赵家沟-川杨河)";
             }
             else if (aisInChannel.getZhaojiagoudong())
             {
                 return "赵家沟东段";
             }
             else if (aisInChannel.getZhaojiagou())
             {
                 return "赵家沟";
             }
        }
        return null;
    }
    /**
     * 在围栏内的开始存储  只存储修改
     * @param inAll
     */
    @Transactional
    void saveAllInData( List<GHAisInfo>  inAll)
    {
        Map<String,GHAisInfo> mapAll=inAll.stream().collect(Collectors.toMap(GHAisInfo::getDeviCeid, Function.identity()));

        List<ShipSealOffInfo> insertNeedList =new ArrayList<>();
        List<ShipSealOffInfo> updateNeedList =new ArrayList<>();
        List<ShipSealOffPoints> sealOffPoints=new ArrayList<>();
        Map<String,BoatInfoEntity> redisDataMap=new HashMap<>();

        mapAll.keySet().forEach(i-> {
            Object result = redisUtilTemplate.getCacheObject(i);
            if (!ObjectUtils.isEmpty(result)) {
                //已经在监控了
                //缓存需更新数据
                BoatInfoEntity redisData = JSON.parseObject((String) result, BoatInfoEntity.class);
                GHAisInfo thisAisPoint = mapAll.get(i);
                List<GHAisInfo> aisInfos = redisData.getAisData();
                String thisGpsTime = DateUtils.dateToString(thisAisPoint.getGpsTime(), DateUtils.DATE_FORMAT_19);
                if (redisData.getLastAisTime().hashCode() != thisGpsTime.hashCode())  //只处理时间不同的
                {
                    //没产生过预警，且已经有一个船速大于1的点位  需要产生预警
                      if (!redisData.getIsWarning() && redisData.getPointWarningNum() > 1 && thisAisPoint.getVelocity().compareTo(new BigDecimal("1")) > 0) {
                                    ShipSealOffInfo info = new ShipSealOffInfo();
                                    info.setSealId(redisData.getId());
                                    info.setMmsi(i);
                                    info.setWarningVerifyStatus("待核实");
                                    info.setWarningTime(new Date());
                                    redisData.setWarningChannel(getInChannelForThisAIS(thisAisPoint));
                                    info.setWarningChannel(redisData.getWarningChannel());
                                    info.setShipBreadth(thisAisPoint.getShipBreadth());
                                    info.setShipLength(thisAisPoint.getShipLength());
                                    info.setShipName(thisAisPoint.getBoatName());
                                    if (thisAisPoint.getVelocity().compareTo(new BigDecimal("1")) > 0) {
                                        info.setWarningStatus("航行中");
                                        redisData.setCunState("航行中");
                                    } else {
                                        info.setWarningStatus("已停泊");
                                        redisData.setCunState("已停泊");
                                    }
                                    info.setStartWarning(redisData.getBeginTime());
                                    insertNeedList.add(info);
                                    List<ShipSealOffPoints> startPoints=new ArrayList<>();
                                    aisInfos.forEach(j -> {
                                        //需补充监控未预警的点位
                                        ShipSealOffPoints sealOffPointsthis = new ShipSealOffPoints();
                                        sealOffPointsthis.setId(snowflakeConfig.snowflakeStringId());
                                        sealOffPointsthis.setMmsi(i);
                                        sealOffPointsthis.setShipName(j.getBoatName());
                                        sealOffPointsthis.setSourceType("市港航AIS");
                                        sealOffPointsthis.setSealId(redisData.getId());
                                        sealOffPointsthis.setAisTime(j.getGpsTime());
                                        sealOffPointsthis.setPointJson(JSON.toJSONString(j));
                                        sealOffPointsthis.setPointGeom(j.getMapx().toString() + " " + j.getMapy().toString());
                                        startPoints.add(sealOffPointsthis);
                                    });
                                    if (startPoints!=null&& startPoints.size()>0) {
                                        log.info("---   该船"+i+"补充开始预警点位："+startPoints.size()+"个，内容："+startPoints.toString());
                                        sealOffPoints.addAll(startPoints);
                                    }
                                      //需增加本次的点位
                                      ShipSealOffPoints sealOffPoints1 = new ShipSealOffPoints();
                                      sealOffPoints1.setId(snowflakeConfig.snowflakeStringId());
                                      sealOffPoints1.setMmsi(i);
                                      sealOffPoints1.setShipName(thisAisPoint.getBoatName());
                                      sealOffPoints1.setSourceType("市港航AIS");
                                      sealOffPoints1.setSealId(redisData.getId());
                                      sealOffPoints1.setAisTime(thisAisPoint.getGpsTime());
                                      sealOffPoints1.setPointJson(JSON.toJSONString(thisAisPoint));
                                      sealOffPoints1.setPointGeom(thisAisPoint.getMapx().toString() + " " + thisAisPoint.getMapy().toString());
                                      sealOffPoints.add(sealOffPoints1);

                                }
                      else {  //全部记录定位 并更新状态
                            if (redisData.getIsWarning()) {
                                ShipSealOffInfo info = new ShipSealOffInfo();
                                info.setSealId(redisData.getId());
                                info.setMmsi(i);
                                info.setWarningVerifyStatus("待核实");
                                if (thisAisPoint.getVelocity().compareTo(new BigDecimal("1")) > 0) {
                                    info.setWarningStatus("航行中");
                                    redisData.setCunState("航行中");
                                } else {
                                    info.setWarningStatus("已停泊");
                                    redisData.setCunState("已停泊");
                                }
                                info.getWarningStatus();
                                updateNeedList.add(info);
                                //需增加本次的点位
                                ShipSealOffPoints sealOffPoints1 = new ShipSealOffPoints();
                                sealOffPoints1.setId(snowflakeConfig.snowflakeStringId());
                                sealOffPoints1.setMmsi(i);
                                sealOffPoints1.setShipName(thisAisPoint.getBoatName());
                                sealOffPoints1.setSourceType("市港航AIS");
                                sealOffPoints1.setSealId(redisData.getId());
                                sealOffPoints1.setAisTime(thisAisPoint.getGpsTime());
                                sealOffPoints1.setPointJson(JSON.toJSONString(thisAisPoint));
                                sealOffPoints1.setPointGeom(thisAisPoint.getMapx().toString() + " " + thisAisPoint.getMapy().toString());
                                sealOffPoints.add(sealOffPoints1);
                            }
                      }
                      if (thisAisPoint.getVelocity().compareTo(new BigDecimal("1")) > 0) {
                           int currNum= redisData.getPointWarningNum();
                            redisData.setPointWarningNum(currNum+1);
                      }
                       redisData.setLastAisTime(DateUtils.dateToString(thisAisPoint.getGpsTime(),DateUtils.DATE_FORMAT_19));
                       aisInfos.add(thisAisPoint); //本次点位添加
                       redisData.setAisData(aisInfos);
                       redisData.setPointNum((redisData.getPointNum() + 1));
                       String dtoJSON = JSON.toJSONString(redisData);
                       // 将JSON字符串存入Redis
                       redisUtilTemplate.setCacheObject(i, dtoJSON);
                 }
                redisDataMap.put(i,redisData);
            }
            else
                {   //还没监控 且船速大于1 头一回列入  只添加到缓存，不做任何处理
                     GHAisInfo thisAisPoint= mapAll.get(i);
                     if (!ObjectUtils.isEmpty(thisAisPoint.getVelocity())){
                         if (thisAisPoint.getVelocity().compareTo(new BigDecimal("1"))>0) {
                             BoatInfoEntity thisAis = new BoatInfoEntity();
                             String id = snowflakeConfig.snowflakeStringId();
                             thisAis.setId(id);
                             thisAis.setPointNum(1);
                             thisAis.setBeginTime(thisAisPoint.getGpsTime());
                             thisAis.setPointWarningNum(1);
                             thisAis.setIsWarning(false);
                             //设置初始航道
                             thisAis.setInChannel(getInChannelForThisAIS(thisAisPoint));
                             List<GHAisInfo> aisInfos = new ArrayList<>();
                             thisAis.setLastAisTime(DateUtils.dateToString(thisAisPoint.getGpsTime(),DateUtils.DATE_FORMAT_19));
                             aisInfos.add(thisAisPoint);
                             thisAis.setAisData(aisInfos);
                             String dtoJSON = JSON.toJSONString(thisAis);
                             redisUtilTemplate.setCacheObject(i, dtoJSON);
                             redisDataMap.put(i,thisAis);
                         }
                     }
            }
        });
        //补充船舶联系数据
        if (insertNeedList!=null && insertNeedList.size()>0)
        {
            log.info("==  3.1 获取预警船舶的联系方式："+insertNeedList.size());
            Map<String,ShipSealOffInfo> sealOffInfoMap= insertNeedList.stream().collect(Collectors.toMap(ShipSealOffInfo::getMmsi,Function.identity()));
             AtomicReference<String> mmsi= new AtomicReference<>("");
            sealOffInfoMap.keySet().forEach(i-> {
                if ("".equals(mmsi.get())){
                    mmsi.set(i);
                }else {
                    mmsi.set(mmsi.get() +","+ i);
                } });
            log.info("==  需要船舶的联系方式的MMSI为："+mmsi.get());
            JSONObject params1 = new JSONObject();
            params1.put("pageIndex", 1);
            params1.put("pageSize", 500);
            params1.put("mmsi", mmsi.get());
            String response = GhjgRpcUtils.getShipLianxi(transUrl, params1);
            if (!StringUtils.isEmpty(response))
            {
                JSONObject object = JSON.parseObject(response);
                JSONArray jsonArray = object.getJSONArray("rows");
                if (jsonArray != null && jsonArray.size() > 0) {
                    List<ShipPhoneForMmsi> phoneForMmsis = new ArrayList<>(jsonArray.size());
                    jsonArray.forEach(i -> {
                        phoneForMmsis.add(JSON.parseObject(i.toString(), ShipPhoneForMmsi.class));
                    });

                    if (phoneForMmsis!=null && phoneForMmsis.size()>0) {
                        log.info("==  获取到的为："+phoneForMmsis.toString());
                        Map<String, ShipPhoneForMmsi> allForInfoShiBie = phoneForMmsis.stream().collect(Collectors.toMap(ShipPhoneForMmsi::getMmsi,Function.identity()));
                        sealOffInfoMap.keySet().forEach(i->{
                            ShipSealOffInfo sealOffInfo=sealOffInfoMap.get(i);
                            ShipPhoneForMmsi phonePeson=allForInfoShiBie.get(i);
                            if (phonePeson!=null)
                            {
                                String nowData=DateUtils.getNowTimeStringToDD(DateUtils.DATE_FORMAT_19);
                                params1.put("shipId",phonePeson.getShipId());
                                params1.put("startTime",DateUtils.dateToString(DateUtils.dayAddNum(new Date(),-5),DateUtils.DATE_FORMAT_19));
                                params1.put("endTime",nowData);
                                String result=GhjgRpcUtils.getShipReportData(transUrl, params1);
                                if (!StringUtils.isEmpty(result)) {
                                    JSONObject object1 = JSON.parseObject(result);
                                    JSONArray jsonArray1 = object1.getJSONArray("rows");
                                    if (jsonArray1 != null && jsonArray1.size() > 0) {
                                        List<ShipReportData> phoneForMmsis1 = new ArrayList<>(jsonArray1.size());
                                        jsonArray1.forEach(j -> {
                                            phoneForMmsis1.add(JSON.parseObject(j.toString(), ShipReportData.class));
                                        });
                                        if (phoneForMmsis1!=null && phoneForMmsis1.size()>0) {
                                            log.info("==  报港数据获取到的为："+phoneForMmsis1.toString());
                                            Map<String, ShipReportData> allForInfoShiBie1 = phoneForMmsis1.stream().collect(Collectors.toMap(ShipReportData::getMmsi,Function.identity()));
                                            sealOffInfoMap.keySet().forEach(j->{
                                                ShipSealOffInfo sealOffInfo1=sealOffInfoMap.get(j);
                                                ShipReportData phonePeson1=allForInfoShiBie1.get(j);
                                                if (phonePeson1!=null)
                                                {
                                                    sealOffInfo1.setShipPhone(phonePeson1.getMobilePhone());
                                                    sealOffInfo1.setShipOwnerCn(phonePeson1.getBidName());
                                                }
                                            });
                                        }
                                    }
                                }else {
                                    sealOffInfo.setShipPhone(phonePeson.getPhone());
                                    sealOffInfo.setShipOwnerCn(phonePeson.getContact());
                                }
                            }
                        });
                    }
                }

            }
        }

        if (insertNeedList!=null && insertNeedList.size()>0) {
            log.info("==  3.2 其中船速大于1连续两个点需要预警的有："+insertNeedList.size()+"个："+insertNeedList.toString());
            shipSealOffInfoService.saveBatch(insertNeedList);
          List<String> strings=  insertNeedList.stream().map(ShipSealOffInfo::getMmsi).collect(Collectors.toList());
            log.info("==  3.2.1  调整缓存点为已预警："+strings.toString());
            strings.stream().forEach(i->{
                Object result = redisUtilTemplate.getCacheObject(i);
                if (!ObjectUtils.isEmpty(result)) {
                    BoatInfoEntity redisData = JSON.parseObject((String) result, BoatInfoEntity.class);
                    redisData.setIsWarning(true);
                    String dtoJSON = JSON.toJSONString(redisData);
                    // 将JSON字符串存入Redis
                    log.info("===   3.2.1.1 开始处理："+i+"设置状态为"+redisData.getIsWarning());
                    redisUtilTemplate.setCacheObject(i, dtoJSON);
                }
            });
            sendSMSData(insertNeedList);
        }
        if (updateNeedList!=null && updateNeedList.size()>0) {
            log.info("==  3.3 其中需要更新船舶："+updateNeedList.size()+"个："+updateNeedList.toString());
            shipSealOffInfoService.updateBatchById(updateNeedList);
        }
        if (sealOffPoints!=null && sealOffPoints.size()>0) {
            log.info("==  3.4 本次新增点位："+sealOffPoints.size()+"个："+sealOffPoints.toString());
            offPointsMapper.saveAll(sealOffPoints,3857);
        }
    }
    /**
     * 在围栏内的开始存储  只存储修改
     * @param notInAll
     */
    void saveAllNotData( List<GHAisInfo>  notInAll)
    {

        Map<String,GHAisInfo> mapAll=notInAll.stream().collect(Collectors.toMap(GHAisInfo::getDeviCeid, Function.identity()));
        log.info("== 离开区域范围内的船舶个数："+notInAll.size()+"，离开的详细信息："+mapAll.keySet());

        List<String> sealIds=new ArrayList<>();

        mapAll.keySet().forEach(i->{
            Object result = redisUtilTemplate.getCacheObject(i);
            if (!ObjectUtils.isEmpty(result)) {  //已经在监控了
                BoatInfoEntity redisData = JSON.parseObject((String) result, BoatInfoEntity.class);
                sealIds.add(redisData.getId());
                redisUtilTemplate.deleteObject(i);
            }
        });
        if(sealIds!=null && sealIds.size()>0) {

            LambdaUpdateWrapper<ShipSealOffInfo> sealOffInfoLambdaUpdateWrapper = new LambdaUpdateWrapper();
            sealOffInfoLambdaUpdateWrapper.set(ShipSealOffInfo::getWarningStatus, "已离开电子围栏");
            sealOffInfoLambdaUpdateWrapper.in(ShipSealOffInfo::getSealId, sealIds);
            shipSealOffInfoService.update(sealOffInfoLambdaUpdateWrapper);

            LambdaQueryWrapper<ShipSealOffInfo> lastPoint = new LambdaQueryWrapper<>();
            lastPoint.in(ShipSealOffInfo::getSealId, sealIds);
            List<ShipSealOffInfo> has = shipSealOffInfoService.list(lastPoint);
            if (has != null && has.size() > 0) {
                Map<String, String> hasMMSI = has.stream().filter(i -> !StringUtils.isEmpty(i.getMmsi())).collect(Collectors.toMap(ShipSealOffInfo::getMmsi, ShipSealOffInfo::getSealId));
                List<ShipSealOffPoints> lastPoints = new ArrayList<>();
                hasMMSI.keySet().forEach(i -> {
                    GHAisInfo aisInfo = mapAll.get(i);
                    ShipSealOffPoints sealOffPoints1 = new ShipSealOffPoints();
                    sealOffPoints1.setId(snowflakeConfig.snowflakeStringId());
                    sealOffPoints1.setMmsi(i);
                    sealOffPoints1.setShipName(aisInfo.getBoatName());
                    sealOffPoints1.setSourceType("市港航AIS");
                    sealOffPoints1.setSealId(hasMMSI.get(i));
                    sealOffPoints1.setAisTime(aisInfo.getGpsTime());
                    sealOffPoints1.setPointJson(JSON.toJSONString(aisInfo));
                    sealOffPoints1.setPointGeom(aisInfo.getMapx().toString() + " " + aisInfo.getMapy().toString());
                    lastPoints.add(sealOffPoints1);
                });
                offPointsMapper.saveAll(lastPoints, 3857);
            }
        }

        }



    /**
     *    1.先查看缓存里的数据在数据库里数据状态是否为待确认，待确认的就保留，其他删除
     */
    void getDBAllDataForNeed()
    {
        Collection<String> cacheKeys  =redisUtilTemplate.keys("*");
        if (cacheKeys!=null && cacheKeys.size()>0) {
            List<String> keys = cacheKeys.stream().collect(Collectors.toList());
            List<String> filteredKeys = keys.stream().filter(key -> !key.contains("_zy"))
                    .collect(Collectors.toList());
            LambdaQueryWrapper<ShipSealOffInfo> lambdaQueryWrapper = new LambdaQueryWrapper<>();
            lambdaQueryWrapper.ne(ShipSealOffInfo::getWarningVerifyStatus, "待核实");
            lambdaQueryWrapper.in(ShipSealOffInfo::getMmsi, filteredKeys);
            lambdaQueryWrapper.eq(ShipSealOffInfo::getDelFlag, "0");
            List<ShipSealOffInfo> shipSealOffInfos= shipSealOffInfoService.list(lambdaQueryWrapper);
            if (shipSealOffInfos!=null && shipSealOffInfos.size()>0)
            {
                List<String> mmsis=shipSealOffInfos.stream().map(ShipSealOffInfo::getMmsi).collect(Collectors.toList());
                log.info("=== 1.1 查询到 存在缓存中的数据已经确认过的数据，开始清除相关缓存，清除船舶为："+mmsis.toString());
                mmsis.forEach(i->{
                    redisUtilTemplate.deleteObject(i);
                });
            }
        }

    }


    /**
     * 短信发送
     */
    void sendSMSData( List<ShipSealOffInfo> inAll){
        inAll.stream().forEach(i->{
            String message="【"+DateUtils.dateToString(i.getWarningTime(),DateUtils.DATE_FORMAT_19)+"】，发现【"+i.getShipName()+"】违规驶入封航区域，请尽快核实。";
            massmsService.commonSend("13393931806",message);
//            massmsService.commonSend("18202163836",message);
//            massmsService.commonSend("13661964733",message);
//            massmsService.commonSend("13818963821",message);
//            massmsService.commonSend("13761118033",message);
//            massmsService.commonSend("19855003202",message);
        });

    }
}
